package client

import (
	"context"
	"sort"

	apiv1 "github.com/acorn-io/acorn/pkg/apis/api.acorn.io/v1"
	apierror "k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	kclient "sigs.k8s.io/controller-runtime/pkg/client"
)

func (c *client) VolumeList(ctx context.Context) (result []apiv1.Volume, _ error) {
	vols := &apiv1.VolumeList{}
	err := c.Client.List(ctx, vols, &kclient.ListOptions{
		Namespace: c.Namespace,
	})
	if err != nil {
		return nil, err
	}

	sort.Slice(vols.Items, func(i, j int) bool {
		if vols.Items[i].CreationTimestamp.Time == vols.Items[j].CreationTimestamp.Time {
			return vols.Items[i].Name < vols.Items[j].Name
		}
		return vols.Items[i].CreationTimestamp.After(vols.Items[j].CreationTimestamp.Time)
	})

	return vols.Items, nil
}

func (c *client) VolumeGet(ctx context.Context, name string) (*apiv1.Volume, error) {
	vol := &apiv1.Volume{}
	return vol, c.Client.Get(ctx, kclient.ObjectKey{
		Name:      name,
		Namespace: c.Namespace,
	}, vol)
}

func (c *client) VolumeDelete(ctx context.Context, name string) (*apiv1.Volume, error) {
	// get first to ensure the namespace matches
	v, err := c.VolumeGet(ctx, name)
	if apierror.IsNotFound(err) {
		return nil, nil
	}
	return v, c.Client.Delete(ctx, &apiv1.Volume{
		ObjectMeta: metav1.ObjectMeta{
			Name:      name,
			Namespace: c.Namespace,
		},
	})
}
